@@ -464,4 +464,96 @@ def ProcessImageTest(image, outputfolder,crop_method = 'Auto', crop_param = None
         axs.axis('off')
         plt.show();
     return df, tiles, masks, corners
+
+## -- Function for parallel processing batches of images 
+   
+def ParallelProcessBatch(sourcefolder, outputfolder, numThreads = 2,   
+                crop_method = 'Auto', s = 200, pin_size = 25, array_dimensions = (8,12),
+                adjust = True, rotate = False, save = True, display = False):
     
+    # -- Define workhorse function with input arguments
+    def workMethod(tno, workQueue):
+        count = 0
+        for item in workQueue:
+            # ... process item ... #
+            print("[Thread %d] Processing item '%s'" % (tno, str(item)))
+            try:
+                Process1Image(str(item), sourcefolder, outputfolder,
+                             corners, s = s, pin_size = pin_size, array_dimensions = array_dimensions,
+                             adjust = adjust, rotate = rotate, save = True, display = False)
+                count = count + 1
+            except:
+                continue
+        print("[Thread %d] Processed '%s' out of '%s' images" % (tno, str(count),str(len(workQueue))))
+        file_object = open(os.path.join(outputfolder,'BatchParameters.txt'), 'a')
+        file_object.write(str(count)+' out of '+ str(len(file_list))+ ' images processed on thread ' + str(tno)+'.')
+        file_object.close()
+        return None
+    
+    # -- Set Up Folders
+    wd = os.getcwd()
+    if save:
+        if not os.path.isdir(outputfolder):
+            os.mkdir(outputfolder)
+    # -- List Files
+    file_list = PP.ListFiles(sourcefolder)
+    image_list = [x  for x in file_list if "jpg" or "JPG" or "JPEG" or "jpeg" in x]
+    
+    # -- Calibrate using first image
+    file = image_list[0]
+    df, tiles, masks, corners = ProcessImage(file, sourcefolder, outputfolder,
+                                     crop_method = crop_method, crop_param = None, s = s, 
+                                     pin_size = pin_size, array_dimensions = array_dimensions,
+                                     adjust = adjust, rotate = rotate, 
+                                     save = False, display = True, calibrate = True)
+    PP.DisplayTiles(tiles)
+    user_input = input("Would you like to proceed? y/n")
+        
+    # -- Process all images
+    if user_input == 'y':
+        lines = (['BATCH PROCESS PARAMETERS',
+                'Crop method: '+crop_method, 
+                'Tile side length: ' +str(s)+ ' pixels', 
+                'Pin radius: '+str(pin_size)+ ' pixels', 
+                'Array dimensions: '+str(array_dimensions),
+                'Autoadjust: '+str(adjust), 
+                'Rotate images: '+str(adjust),
+                'SEGMENTATION PARAMETERS',
+                'Minimum intensity: 0.035 + background',
+                'Minimum area: np.pi*(pin_size+10)**2',
+                'Minimum circle certainty: 0.42',
+                'Maximum circle certainty: 0.92', 
+                'CROPPING PARAMETERS',
+                'Min object size in auto adjust: 2000 pixels',
+                'Plate Height in auto crop: ~1700 pixels',
+                'Plate intensity in auto crop: 0.5',
+                'Row buffer in auto crop: 250',
+                'Column buffer in auto crop: 175',
+                'COMPLETION'])
+        
+        with open(os.path.join(outputfolder,'BatchParameters.txt'), 'w') as f:
+            for line in lines:
+                f.write(line)
+                f.write('\n')
+                
+        ## Break image_list up into chunks for each thread and assign workMethod
+        threads = []
+        for tno in range(0, numThreads-1):
+            # divide the allWork among the threads, and make sure not to miss anything ...
+            chunkSize = int(len(image_list)/numThreads)
+            workStart = chunkSize*tno
+            workEnd = chunkSize*(tno+1)
+            threads.append(threading.Thread(target=workMethod, args=(tno, image_list[workStart:workEnd])))
+            threads[-1].start()
+        # ... last thread takes the leftovers    
+        workStart = chunkSize*(numThreads-1)
+        workEnd = len(image_list)
+        threads.append(threading.Thread(target=workMethod, args=(numThreads-1, image_list[workStart:workEnd])))
+        threads[-1].start()
+            
+
+        # wait for each thread to finish its work, the script will not progress until all work is done
+        for thd in threads:
+            thd.join()
+
+    return None
\ No newline at end of file
